
#############################################################################
# Directories for Installation
#############################################################################

ifeq (${COMPILE_OPTION}, debug)
	OBJ	= objd
	DFLAGS  = -g  
	LDFLAGS = $(DFLAGS) $(P_LDFLAGS) -L${O} 
else
	OBJ	= obj
	DFLAGS  = 
	LDFLAGS = $(DFLAGS) $(P_LDFLAGS) -L${O} -Xlinker -s
endif

O=${OBJ}/i386

#-- location of lib3des.a
TRIPLEDESDIR   = ../libs/${COMPILE_OPTION}/${MWCONFIG_NAME}
BINDIR      = ./bin

#############################################################################
# Verbosity Control in builds
# Information below taken from /vobs/mw/mw/make.noise
#############################################################################
ifndef MAKE_VERBOSE
NO_SHORT_ECHO = TRUE
endif
 
ifdef NO_SHORT_ECHO
 
ECHOOBJ    = @
ECHOTARGET = @
ECHOSOURCE = @
ECHOLIB    = @
ECHONOISE  = @true
 
else
 
  ifndef DIRNOISE
    DIRNOISE := $(shell pwd)
  endif
 
ECHOOBJ    = @echo "[ ${DIRNOISE} : $(notdir $<) --> $(notdir $@) ]"
ECHOTARGET = @echo "[ ${DIRNOISE} : $(notdir $@) ]"
ECHOSOURCE = @echo "[ ${DIRNOISE} : $(notdir $<) ]"
ECHOLIB    = @echo "[ -- ${DIRNOISE} : $(notdir $@) ]"
ECHONOISE  = @echo
 
endif
 
ifeq "$(strip ${MAKE_VERBOSE})"  "false"
.SILENT:
else
DASH_VERBOSE=-v
endif

#############################################################################

CFILES		= \
              sso_pakt.cpp  \
              ssod.cpp        \
              password.cpp    \
              dimonfnc.cpp \
              crypto.cpp      \
              sha1.c          \
              rotlr.c

HFILES		= \
              sso_pakt.h  \
              password.h    \
              dimonfnc.h \
              crypto.h

OFILES		= \
              ${O}/ssod.o        \
              ${O}/sso_pakt.o  \
              ${O}/password.o    \
              ${O}/dimonfnc.o \
              ${O}/crypto.o      \
              ${O}/sha1.o        \
              ${O}/rotlr.o

EXE         = ssod.${EXTENSION}

EXE.arch   = ${EXE:%=${BINDIR}/%}

#############################################################################
# Platform specific information
#############################################################################

IFLAGS    =  -I /usr/include


EXTRALIBS = ${P_LIBS} -L${STDCLIBDIR} -lstdc++ -L${GNULIBDIR} -lgcc -L${TRIPLEDESDIR} -l3des

CFLAGS    = $(OPTFLAGS) $(DFLAGS) $(P_CFLAGS) $(IFLAGS)   $(PLATFORM_CFLAGS)

#############################################################################
# G++ directives
#############################################################################
.SUFFIXES: .C
.C.o:
	$(CC) $(CFLAGS) -c $<x
#############################################################################

all: alldirs ${EXE.arch}

alldirs: ${O}  ${BINDIR}

${EXE.arch}: $(OFILES) 
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(LDFLAGS) -o $@ $(OFILES) ${EXTRALIBS} $(PLATFORM_SPECIFIC_LIBS)

clean:
	rm -f $(O)/* *~ core

realclean: clean
	rm -rf $(EXE) Templates.DB

${O}/ssod2pam.o : ssod2pam.cpp ssod2pam.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $< 

${O}/ssod.o : ssod.cpp password.h sso_pakt.h dimonfnc.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $< 

${O}/password.o : password.cpp password.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/sso_pakt.o : sso_pakt.cpp sso_pakt.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/dimonfnc.o : dimonfnc.cpp dimonfnc.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/crypto.o : crypto.cpp crypto.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/sha1.o : sha1.c sha.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/rotlr.o : rotlr.c
	$(CC) $(CFLAGS) -c -o $@ $<

${O} ${BINDIR}:
	${ECHONOISE} "[ $@ ]"
	mkdir -p $@

